import { useState, useEffect } from 'react';
var observer;
var isClient = typeof window !== 'undefined';

var hasIntersectionObserverSupport = function hasIntersectionObserverSupport() {
  return typeof window.IntersectionObserver !== 'undefined';
}; // import intersection observer polyfill only if on client
// and if not supported by browser


var getIntersectionObserver = function getIntersectionObserver() {
  return Promise.resolve(isClient && !hasIntersectionObserverSupport() && import('intersection-observer'));
}; // options for intersection observer in order to improve the effect


var intersectionObserverOptions = {
  root: null,
  // windows viewport will be used
  rootMargin: '200px 0px 0px 0px'
};

var handleIntersect = function handleIntersect(entries, observer) {
  entries.filter(function (entry) {
    return entry.isIntersecting;
  }).forEach(function (entry) {
    var target = entry.target;

    target._onIntersect(observer);
  });
}; // get observer from the cache or create a new one


var getObserver = function getObserver() {
  return getIntersectionObserver().then(function (_) {
    if (observer) return observer;

    if (isClient) {
      return new window.IntersectionObserver(handleIntersect, intersectionObserverOptions);
    }
  });
};

export default (function (_ref) {
  var ref = _ref.ref;

  var _useState = useState(false),
      show = _useState[0],
      setShow = _useState[1];

  useEffect(function () {
    var current = ref.current;
    if (!current) return; // get observer async and only when needed

    getObserver().then(function (observer) {
      return observer.observe(current);
    }); // mutate the current target to add a method to be executed when intersecting

    current._onIntersect = function (observer) {
      setShow(true);
      observer.unobserve(current);
    }; // clean observed element when unmounted


    return function () {
      return current._isSubscribed = false;
    };
  }, [ref]);
  return show;
});